@@ -39,24 +39,8 @@
  * | string length | yes | `char *` | string JSON format of the XPath expression |
  */
 
-/**
- * @brief Print xpath1.0 token in the specific format.
- *
- * @param[in] token Token to transform.
- * @param[in] tok_len Lenghth of @p token.
- * @param[in] is_nametest Whether the token is a nametest, it then always requires a prefix in XML @p get_format.
- * @param[in,out] context_mod Current context module, may be updated.
- * @param[in] resolve_ctx Context to use for resolving prefixes.
- * @param[in] resolve_format Format of the resolved prefixes.
- * @param[in] resolve_prefix_data Resolved prefixes prefix data.
- * @param[in] get_format Format of the output prefixes.
- * @param[in] get_prefix_data Format-specific prefix data for the output.
- * @param[out] token_p Printed token.
- * @param[out] err Error structure on error.
- * @return LY_ERR value.
- */
-static LY_ERR
-xpath10_print_token(const char *token, uint16_t tok_len, ly_bool is_nametest, const struct lys_module **context_mod,
+LIBYANG_API_DEF LY_ERR
+lyplg_type_xpath10_print_token(const char *token, uint16_t tok_len, ly_bool is_nametest, const struct lys_module **context_mod,
         const struct ly_ctx *resolve_ctx, LY_VALUE_FORMAT resolve_format, const void *resolve_prefix_data,
         LY_VALUE_FORMAT get_format, void *get_prefix_data, char **token_p, struct ly_err_item **err)
 {
@@ -180,7 +164,7 @@ xpath10_print_subexpr_r(uint16_t *cur_idx, enum lyxp_token end_tok, const struct
         if ((cur_tok == LYXP_TOKEN_NAMETEST) || (cur_tok == LYXP_TOKEN_LITERAL)) {
             /* tokens that may include prefixes, get them in the target format */
             is_nt = (cur_tok == LYXP_TOKEN_NAMETEST) ? 1 : 0;
-            LY_CHECK_RET(xpath10_print_token(cur_exp_ptr, xp_val->exp->tok_len[*cur_idx], is_nt, &context_mod,
+            LY_CHECK_RET(lyplg_type_xpath10_print_token(cur_exp_ptr, xp_val->exp->tok_len[*cur_idx], is_nt, &context_mod,
                     xp_val->ctx, xp_val->format, xp_val->prefix_data, format, prefix_data, &str_tok, err));
 
             /* append the converted token */
@@ -232,18 +216,8 @@ error_mem:
     return ly_err_new(err, LY_EMEM, LYVE_DATA, NULL, NULL, "No memory.");
 }
 
-/**
- * @brief Print xpath1.0 value in the specific format.
- *
- * @param[in] xp_val xpath1.0 value structure.
- * @param[in] format Format to print in.
- * @param[in] prefix_data Format-specific prefix data.
- * @param[out] str_value Printed value.
- * @param[out] err Error structure on error.
- * @return LY_ERR value.
- */
-static LY_ERR
-xpath10_print_value(const struct lyd_value_xpath10 *xp_val, LY_VALUE_FORMAT format, void *prefix_data,
+LIBYANG_API_DEF LY_ERR
+lyplg_type_print_xpath10_value(const struct lyd_value_xpath10 *xp_val, LY_VALUE_FORMAT format, void *prefix_data,
         char **str_value, struct ly_err_item **err)
 {
     LY_ERR ret = LY_SUCCESS;
@@ -251,12 +225,14 @@ xpath10_print_value(const struct lyd_value_xpath10 *xp_val, LY_VALUE_FORMAT form
     uint32_t str_len = 0;
 
     *str_value = NULL;
+    *err = NULL;
 
     /* recursively print the expression */
     ret = xpath10_print_subexpr_r(&expr_idx, 0, NULL, xp_val, format, prefix_data, str_value, &str_len, err);
 
     if (ret) {
         free(*str_value);
+        *str_value = NULL;
     }
     return ret;
 }
@@ -327,7 +303,7 @@ lyplg_type_store_xpath10(const struct ly_ctx *ctx, const struct lysc_type *type,
     case LY_VALUE_SCHEMA_RESOLVED:
     case LY_VALUE_XML:
         /* JSON format with prefix is the canonical one */
-        ret = xpath10_print_value(val, LY_VALUE_JSON, NULL, &canon, err);
+        ret = lyplg_type_print_xpath10_value(val, LY_VALUE_JSON, NULL, &canon, err);
         LY_CHECK_GOTO(ret, cleanup);
 
         ret = lydict_insert_zc(ctx, canon, &storage->_canonical);
@@ -349,6 +325,49 @@ cleanup:
     return ret;
 }
 
+/**
+ * @brief Create a namespace and add it into a set.
+ *
+ * @param[in] set Set of namespaces to add to.
+ * @param[in] pref Namespace prefix.
+ * @param[in] uri Namespace URI.
+ * @return LY_ERR value.
+ */
+static LY_ERR
+xpath10_add_ns(struct ly_set *set, const char *pref, const char *uri)
+{
+    LY_ERR rc = LY_SUCCESS;
+    struct lyxml_ns *ns = NULL;
+
+    /* create new ns */
+    ns = calloc(1, sizeof *ns);
+    if (!ns) {
+        rc = LY_EMEM;
+        goto cleanup;
+    }
+    ns->prefix = strdup(pref);
+    ns->uri = strdup(uri);
+    if (!ns->prefix || !ns->uri) {
+        rc = LY_EMEM;
+        goto cleanup;
+    }
+    ns->depth = 1;
+
+    /* add into the XML namespace set */
+    if ((rc = ly_set_add(set, ns, 1, NULL))) {
+        goto cleanup;
+    }
+    ns = NULL;
+
+cleanup:
+    if (ns) {
+        free(ns->prefix);
+        free(ns->uri);
+        free(ns);
+    }
+    return rc;
+}
+
 /**
  * @brief Implementation of ::lyplg_type_validate_clb for the xpath1.0 ietf-yang-types type.
  */
@@ -362,7 +381,6 @@ lyplg_type_validate_xpath10(const struct ly_ctx *UNUSED(ctx), const struct lysc_
     struct ly_set *set = NULL;
     uint32_t i;
     const char *pref, *uri;
-    struct lyxml_ns *ns;
 
     *err = NULL;
     LYD_VALUE_GET(storage, val);
@@ -395,28 +413,8 @@ lyplg_type_validate_xpath10(const struct ly_ctx *UNUSED(ctx), const struct lysc_
         assert(!strcmp(LYD_NAME(lyd_child(set->dnodes[i])->next), "uri"));
         uri = lyd_get_value(lyd_child(set->dnodes[i])->next);
 
-        /* create new ns */
-        ns = calloc(1, sizeof *ns);
-        if (!ns) {
-            ret = LY_EMEM;
-            goto cleanup;
-        }
-        ns->prefix = strdup(pref);
-        ns->uri = strdup(uri);
-        if (!ns->prefix || !ns->uri) {
-            free(ns->prefix);
-            free(ns->uri);
-            free(ns);
-            ret = LY_EMEM;
-            goto cleanup;
-        }
-        ns->depth = 1;
-
-        /* add into the XML namespace set */
-        if ((ret = ly_set_add(val->prefix_data, ns, 1, NULL))) {
-            free(ns->prefix);
-            free(ns->uri);
-            free(ns);
+        /* new NS */
+        if ((ret = xpath10_add_ns(val->prefix_data, pref, uri))) {
             goto cleanup;
         }
     }
@@ -455,7 +453,7 @@ lyplg_type_print_xpath10(const struct ly_ctx *ctx, const struct lyd_value *value
     }
 
     /* print in the specific format */
-    if (xpath10_print_value(val, format, prefix_data, &ret, &err)) {
+    if (lyplg_type_print_xpath10_value(val, format, prefix_data, &ret, &err)) {
         if (err) {
             LOGVAL_ERRITEM(ctx, err);
             ly_err_free(err);
@@ -488,7 +486,7 @@ lyplg_type_dup_xpath10(const struct ly_ctx *ctx, const struct lyd_value *origina
     dup_val->ctx = ctx;
 
     LYD_VALUE_GET(original, orig_val);
-    ret = lyxp_expr_dup(ctx, orig_val->exp, &dup_val->exp);
+    ret = lyxp_expr_dup(ctx, orig_val->exp, 0, 0, &dup_val->exp);
     LY_CHECK_GOTO(ret, cleanup);
 
     ret = lyplg_type_prefix_data_dup(ctx, orig_val->format, orig_val->prefix_data, &dup_val->prefix_data);
