@@ -74,7 +74,7 @@ lyht_dup_inst_ht_equal_cb(void *val1_p, void *val2_p, ly_bool mod, void *UNUSED(
  * @return Instance cache entry.
  */
 static struct lyd_dup_inst *
-lyd_dup_inst_get(const struct lyd_node *first_inst, struct hash_table **dup_inst_ht)
+lyd_dup_inst_get(const struct lyd_node *first_inst, struct ly_ht **dup_inst_ht)
 {
     struct lyd_dup_inst **item_p, *item;
 
@@ -102,7 +102,7 @@ lyd_dup_inst_get(const struct lyd_node *first_inst, struct hash_table **dup_inst
 }
 
 LY_ERR
-lyd_dup_inst_next(struct lyd_node **inst, const struct lyd_node *siblings, struct hash_table **dup_inst_ht)
+lyd_dup_inst_next(struct lyd_node **inst, const struct lyd_node *siblings, struct ly_ht **dup_inst_ht)
 {
     struct lyd_dup_inst *dup_inst;
 
@@ -151,7 +151,7 @@ lyht_dup_inst_ht_free_cb(void *val_p)
 }
 
 void
-lyd_dup_inst_free(struct hash_table *dup_inst_ht)
+lyd_dup_inst_free(struct ly_ht *dup_inst_ht)
 {
     lyht_free(dup_inst_ht, lyht_dup_inst_ht_free_cb);
 }
@@ -304,12 +304,21 @@ lyd_owner_module(const struct lyd_node *node)
         opaq = (struct lyd_node_opaq *)node;
         switch (opaq->format) {
         case LY_VALUE_XML:
-            return opaq->name.module_ns ? ly_ctx_get_module_implemented_ns(LYD_CTX(node), opaq->name.module_ns) : NULL;
+            if (opaq->name.module_ns) {
+                return ly_ctx_get_module_implemented_ns(LYD_CTX(node), opaq->name.module_ns);
+            }
+            break;
         case LY_VALUE_JSON:
-            return opaq->name.module_name ? ly_ctx_get_module_implemented(LYD_CTX(node), opaq->name.module_name) : NULL;
+            if (opaq->name.module_name) {
+                return ly_ctx_get_module_implemented(LYD_CTX(node), opaq->name.module_name);
+            }
+            break;
         default:
             return NULL;
         }
+
+        /* try a parent */
+        return lyd_owner_module(lyd_parent(node));
     }
 
     return lysc_owner_module(node->schema);
@@ -498,7 +507,7 @@ lys_value_validate(const struct ly_ctx *ctx, const struct lysc_node *node, const
     struct lyd_value storage;
     struct lysc_type *type;
 
-    LY_CHECK_ARG_RET(ctx, node, value, LY_EINVAL);
+    LY_CHECK_ARG_RET(ctx, node, LY_EINVAL);
 
     if (!(node->nodetype & (LYS_LEAF | LYS_LEAFLIST))) {
         LOGARG(ctx, node);
@@ -1149,9 +1158,9 @@ lyd_find_sibling_schema(const struct lyd_node *siblings, const struct lysc_node
     parent = siblings->parent;
     if (parent && parent->schema && parent->children_ht) {
         /* calculate our hash */
-        hash = dict_hash_multi(0, schema->module->name, strlen(schema->module->name));
-        hash = dict_hash_multi(hash, schema->name, strlen(schema->name));
-        hash = dict_hash_multi(hash, NULL, 0);
+        hash = lyht_hash_multi(0, schema->module->name, strlen(schema->module->name));
+        hash = lyht_hash_multi(hash, schema->name, strlen(schema->name));
+        hash = lyht_hash_multi(hash, NULL, 0);
 
         /* use special hash table function */
         ht_cb = lyht_set_cb(parent->children_ht, lyd_hash_table_schema_val_equal);
@@ -1536,7 +1545,12 @@ ly_format2str(LY_VALUE_FORMAT format)
 LIBYANG_API_DEF int
 ly_time_tz_offset(void)
 {
-    const time_t epoch_plus_11h = 60 * 60 * 11;
+    return ly_time_tz_offset_at(time(NULL));
+}
+
+LIBYANG_API_DEF int
+ly_time_tz_offset_at(time_t time)
+{
     struct tm tm_local, tm_utc;
     int result = 0;
 
@@ -1544,8 +1558,23 @@ ly_time_tz_offset(void)
     tzset();
 
     /* get local and UTC time */
-    localtime_r(&epoch_plus_11h, &tm_local);
-    gmtime_r(&epoch_plus_11h, &tm_utc);
+    localtime_r(&time, &tm_local);
+    gmtime_r(&time, &tm_utc);
+
+    /* account for year/month/day change by adding/subtracting from the hours, the change cannot be more than 1 day */
+    if (tm_local.tm_year < tm_utc.tm_year) {
+        tm_utc.tm_hour += 24;
+    } else if (tm_local.tm_year > tm_utc.tm_year) {
+        tm_local.tm_hour += 24;
+    } else if (tm_local.tm_mon < tm_utc.tm_mon) {
+        tm_utc.tm_hour += 24;
+    } else if (tm_local.tm_mon > tm_utc.tm_mon) {
+        tm_local.tm_hour += 24;
+    } else if (tm_local.tm_mday < tm_utc.tm_mday) {
+        tm_utc.tm_hour += 24;
+    } else if (tm_local.tm_mday > tm_utc.tm_mday) {
+        tm_local.tm_hour += 24;
+    }
 
     /* hours shift in seconds */
     result += (tm_local.tm_hour - tm_utc.tm_hour) * 3600;
@@ -1639,8 +1668,8 @@ ly_time_time2str(time_t time, const char *fractions_s, char **str)
         return LY_ESYS;
     }
 
-    /* get timezone offset */
-    zonediff_s = ly_time_tz_offset();
+    /* get timezone offset (do not use tm_gmtoff to avoid portability problems) */
+    zonediff_s = ly_time_tz_offset_at(time);
     zonediff_h = zonediff_s / 60 / 60;
     zonediff_m = zonediff_s / 60 % 60;
     sprintf(zoneshift, "%+03d:%02d", zonediff_h, zonediff_m);
