@@ -350,19 +350,18 @@ lysp_type_find(const char *id, struct lysp_node *start_node, const struct lysp_m
  * @param[in,out] ctx Context to log the error.
  * @param[in,out] ht Hash table with top-level names.
  * @param[in] name Inserted top-level identifier.
- * @param[in] statement The name of the statement type from which
- * @p name originated (eg typedef, feature, ...).
+ * @param[in] statement The name of the statement type from which @p name originated (eg typedef, feature, ...).
  * @param[in] err_detail Optional error specification.
  * @return LY_ERR, but LY_EEXIST is mapped to LY_EVALID.
  */
 static LY_ERR
-lysp_check_dup_ht_insert(struct lysp_ctx *ctx, struct hash_table *ht,
-        const char *name, const char *statement, const char *err_detail)
+lysp_check_dup_ht_insert(struct lysp_ctx *ctx, struct ly_ht *ht, const char *name, const char *statement,
+        const char *err_detail)
 {
     LY_ERR ret;
     uint32_t hash;
 
-    hash = dict_hash(name, strlen(name));
+    hash = lyht_hash(name, strlen(name));
     ret = lyht_insert(ht, &name, hash, NULL);
     if (ret == LY_EEXIST) {
         if (err_detail) {
@@ -388,7 +387,7 @@ lysp_check_dup_ht_insert(struct lysp_ctx *ctx, struct hash_table *ht,
  */
 static LY_ERR
 lysp_check_dup_typedef(struct lysp_ctx *ctx, struct lysp_node *node, const struct lysp_tpdf *tpdf,
-        struct hash_table *tpdfs_global)
+        struct ly_ht *tpdfs_global)
 {
     struct lysp_node *parent;
     uint32_t hash;
@@ -434,7 +433,7 @@ lysp_check_dup_typedef(struct lysp_ctx *ctx, struct lysp_node *node, const struc
 
     /* check collision with the top-level typedefs */
     if (node) {
-        hash = dict_hash(name, name_len);
+        hash = lyht_hash(name, name_len);
         if (!lyht_find(tpdfs_global, &name, hash, NULL)) {
             LOGVAL_PARSER(ctx, LYVE_SYNTAX_YANG,
                     "Duplicate identifier \"%s\" of typedef statement - scoped type collide with a top-level type.", name);
@@ -469,7 +468,7 @@ lysp_id_cmp(void *val1, void *val2, ly_bool UNUSED(mod), void *UNUSED(cb_data))
 LY_ERR
 lysp_check_dup_typedefs(struct lysp_ctx *ctx, struct lysp_module *mod)
 {
-    struct hash_table *ids_global;
+    struct ly_ht *ids_global;
     const struct lysp_tpdf *typedefs;
     LY_ARRAY_COUNT_TYPE u, v;
     uint32_t i;
@@ -528,7 +527,7 @@ lysp_grouping_match(const char *name, struct lysp_node *node)
  */
 static LY_ERR
 lysp_check_dup_grouping(struct lysp_ctx *ctx, struct lysp_node *node, const struct lysp_node_grp *grp,
-        struct hash_table *grps_global)
+        struct ly_ht *grps_global)
 {
     struct lysp_node *parent;
     uint32_t hash;
@@ -567,7 +566,7 @@ lysp_check_dup_grouping(struct lysp_ctx *ctx, struct lysp_node *node, const stru
 
     /* check collision with the top-level groupings */
     if (node) {
-        hash = dict_hash(name, name_len);
+        hash = lyht_hash(name, name_len);
         if (!lyht_find(grps_global, &name, hash, NULL)) {
             LOGVAL_PARSER(ctx, LYVE_SYNTAX_YANG,
                     "Duplicate identifier \"%s\" of grouping statement - scoped grouping collide with a top-level grouping.", name);
@@ -584,7 +583,7 @@ lysp_check_dup_grouping(struct lysp_ctx *ctx, struct lysp_node *node, const stru
 LY_ERR
 lysp_check_dup_groupings(struct lysp_ctx *ctx, struct lysp_module *mod)
 {
-    struct hash_table *ids_global;
+    struct ly_ht *ids_global;
     const struct lysp_node_grp *groupings, *grp_iter;
     LY_ARRAY_COUNT_TYPE u;
     uint32_t i;
@@ -626,7 +625,7 @@ LY_ERR
 lysp_check_dup_features(struct lysp_ctx *ctx, struct lysp_module *mod)
 {
     LY_ARRAY_COUNT_TYPE u;
-    struct hash_table *ht;
+    struct ly_ht *ht;
     struct lysp_feature *f;
     LY_ERR ret = LY_SUCCESS;
 
@@ -658,7 +657,7 @@ LY_ERR
 lysp_check_dup_identities(struct lysp_ctx *ctx, struct lysp_module *mod)
 {
     LY_ARRAY_COUNT_TYPE u;
-    struct hash_table *ht;
+    struct ly_ht *ht;
     struct lysp_ident *i;
     LY_ERR ret = LY_SUCCESS;
 
@@ -1184,7 +1183,6 @@ lysp_inject_submodule(struct lysp_ctx *pctx, struct lysp_include *inc)
         DUP_STRING_RET(PARSER_CTX(pctx), inc->name, inc_new->name);
         DUP_STRING_RET(PARSER_CTX(pctx), inc->dsc, inc_new->dsc);
         DUP_STRING_RET(PARSER_CTX(pctx), inc->ref, inc_new->ref);
-        /* TODO duplicate extensions */
         memcpy(inc_new->rev, inc->rev, LY_REV_SIZE);
         inc_new->injected = 1;
     }
