@@ -1250,7 +1250,7 @@ lyd_validate_unique(const struct lyd_node *first, const struct lysc_node *snode,
     ly_bool dyn;
     const void *hash_key;
     struct lyd_val_uniq_arg arg, *args = NULL;
-    struct hash_table **uniqtables = NULL;
+    struct ly_ht **uniqtables = NULL;
     struct lyd_value *val;
     struct ly_ctx *ctx = snode->module->ctx;
 
@@ -1307,7 +1307,7 @@ lyd_validate_unique(const struct lyd_node *first, const struct lysc_node *snode,
 
                     /* get hash key */
                     hash_key = val->realtype->plugin->print(NULL, val, LY_VALUE_LYB, NULL, &dyn, &key_len);
-                    hash = dict_hash_multi(hash, hash_key, key_len);
+                    hash = lyht_hash_multi(hash, hash_key, key_len);
                     if (dyn) {
                         free((void *)hash_key);
                     }
@@ -1318,7 +1318,7 @@ lyd_validate_unique(const struct lyd_node *first, const struct lysc_node *snode,
                 }
 
                 /* finish the hash value */
-                hash = dict_hash_multi(hash, NULL, 0);
+                hash = lyht_hash_multi(hash, NULL, 0);
 
                 /* insert into the hashtable */
                 ret = lyht_insert(uniqtables[u], &set->objs[i], hash, NULL);
